"""
Setup script for Dasein.

For most use cases, you should install via:
    pip install .

This setup.py exists for backwards compatibility.
The project is primarily configured via pyproject.toml.
"""

from setuptools import setup

# All configuration is in pyproject.toml
setup()

